# me2/socklib/xmpp/errors.py
#
#

""" xmpp error codes. """

xmpperrors = {'400': 'Bad Request',
              '401': 'Unauthorized',
              '402': 'Payment Required',
              '403': 'Forbidden',
              '404': 'Not Found',
              '405': 'Not Allowed',
              '406': 'Not Acceptable',
              '407': 'Registration Required',
              '408': 'Request Timeout',
              '409': 'Conflict',
              '500': 'Internal Server Error',
              '501': 'Not Implemented',
              '502': 'Remove Server Error',
              '503': 'Service Unavailable',
              '504': 'Remove Server Timeout',
              '510': 'Disconnected'
             }
