<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <link rel="stylesheet" href="/assets/style.css" type="text/css" />
    <link rel="stylesheet" href="/assets/openid.css" type="text/css" />
    <script type="text/javascript" src="assets/jquery-1.2.6.min.js"></script>
    <script type="text/javascript" src="assets/openid-jquery.js"></script>
    <script type="text/javascript">
        $(document).ready(function() {
            openid.init('openid_identifier');
        });
    </script>
</head>
<body>
    <title>Login with Openid</title>
    <div class="related">
      <ul> 
        <li class="right" style="margin-right: 10px"><b><i>the JSON everywhere bot<b></i></li>
        <li><b><i>Welcome to {{ appname }}<b></i></li>
      </ul>
    </div> 
    <div class="footer" id="info_div"><b>login page</b></div>
    <div class="body" align="center">
        <div><h1>{{ appname }}</h1></div><br>
        <div id="topper_div"></div>
        <form action="/_ah/login_required" method="post" id="openid_form">
	    <input type="hidden" name="cont" value="continue" />
	    <fieldset>
    		<legend>Sign-in or Create New Account</legend>
    		<div id="openid_choice">
	    		<p>Please click your account provider:</p>
	    		<div id="openid_btns"></div>
			</div>
			
			<div id="openid_input_area">
				<input id="openid_identifier" name="openid" type="text" value="http://" />
				<input id="openid_submit" type="submit" value="Sign-In"/>
			</div>
	    </fieldset>
        </form>
    </div>
    <div class="footer" id="status_div"></div>
    <div class="footer" id="doc_div"><a href="http://botme2.appspot.com/docs/html/index.html" onclick='window.open("http://botme2.appspot.com/docs/html/index.html"); return false'><i><b>DOCS</b></i></a></div>
    <div class="footer" id="bottom_div"></div>
</body>
</html>